SUBROUTINE CALIBRATE(URSICode, EpochTime, METHOD, MASKRX,  PRI)
!
! Calibrate the raw data complex I/Q values in the PRI using the indicated METHOD
!
! 08 July 2012
! Terry Bullett   University of Colorado   "To God be the Glory"
! 
! This routine assumes that the SCT and PCT have already been read 
! and the relevant elements loaded into the PRI structure
!
!
IMPLICIT NONE
INCLUDE 'pct.f90'
INCLUDE 'pri.f90'
!
!
CHARACTER(LEN=5),INTENT(IN):: URSICode
INTEGER(KIND=4),INTENT(IN) :: EpochTime
INTEGER,INTENT(IN) :: METHOD,MASKRX
TYPE(PRItype),INTENT(INOUT) :: PRI
!
! Local Variables
!
REAL,PARAMETER:: PI=4.0*ATAN(1.0)
REAL,PARAMETER:: c=299792458.0
COMPLEX,PARAMETER :: CZERO=(0.0, 0.0)
!
INTEGER IRX,RGT
REAL,DIMENSION(8):: CableDiff
COMPLEX:: CALFACTOR,CALEXP
!
!
! METHOD=0 means do nothing.
IF (METHOD.EQ.0) RETURN
!
! Determine the calibration factors 
!
SELECT CASE (URSICode) 
CASE('XX000')    !    Default station code
   CableDiff=0.0
CASE('AA109')    ! Addis Ababa
!  07Jul12 
!  Pair-wise comparisons show that Rx1 has the shortest cable, #2 is +4.2m, #5 is +8.3m and #6 is +10.0m
   CableDiff(1) =   0.0 
   CableDiff(2) =  +5.0
   CableDiff(3) =   0.0
   CableDiff(4) =   0.0
   CableDiff(5) =  +6.7
   CableDiff(6) = +10.0
   CableDiff(7) =   0.0
   CableDiff(8) =   0.0
CASE('WI937')    ! Wallops
   CableDiff=0.0
CASE DEFAULT
   CableDiff=0.0
END SELECT

! Calibrate the data
DO IRX = 1, PRI%ReceiverCount
   IF (.NOT.BTEST(MASKRX, IRX-1)) CYCLE   ! Dont bother with masked out receivers
!  Ignore amplitude/attenuation factor for now.
!  Apply the k=2*Pi*f/c factor to the imaginary part of CableDiff.
!  Requires real part to be zero or you will incorrectly attenuate your signal
!  ToDo:  Allow amplitude adjustment based on cable length
   CALEXP = COMPLEX( 0.0 ,  (2.0*PI*PRI%Frequency*1000.0*CableDiff(IRX)/c) ) 
   CALFACTOR=EXP(-CALEXP)
!db   write(*,*) CABLEDIFF(IRX), CALEXP, CALFACTOR, PRI%Frequency 
   DO RGT = 1, PRI%GateCount
      PRI%Ascan(IRX,RGT) = CALFACTOR*PRI%Ascan(IRX,RGT)
   ENDDO
ENDDO

RETURN
! 100 FORMAT (I6,1X,F6.1,I3,1X,2I8,2X,2F8.1)
END SUBROUTINE CALIBRATE 
