#!/bin/sh
#
# calplot -- plot amplitude and phase calibrations from VIPIR
# T. Bullett  Univerity of Colorado.  March 2013
#
RIQFILE=$1
RG=15
PLOTFILE="$HOME/tmp/calplot.gnu"
DATFILE="$(dirname $RIQFILE)/$(basename $RIQFILE .RIQ)_cal.dat"
IMGFILE="$(dirname $RIQFILE)/$(basename $RIQFILE .RIQ)_cal.png"
echo $DATFILE  $IMGFILE 

riq_ionogram -s -G $RG -n1 1 -n2 20000 $RIQFILE > $DATFILE
cat <<EOF  > $PLOTFILE
set title "Calibration data for $RIQFILE"
set xlabel "Frequency [MHz]"
#set logscale x
set xrange [0:30]
set xtics 2
set ylabel "Amplitude [dB]"
set ytics 10
set mytics 10 
set yrange [20:60]
set y2label "Phase [Deg]"
set y2range [-40:40]
set y2tics 5
set grid 
set key outside right
set terminal png size 1280,1024
set output "$IMGFILE"
set pointsize 0.7
plot "$DATFILE" using 6:9  with points pt 1 lc 1 title "A1",\
     "$DATFILE" using 6:13 with points pt 2 lc 2 title "A2",\
     "$DATFILE" using 6:17 with points pt 3 lc 3 title "A3",\
     "$DATFILE" using 6:21 with points pt 4 lc 4 title "A4",\
     "$DATFILE" using 6:25 with points pt 5 lc 5 title "A5",\
     "$DATFILE" using 6:29 with points pt 6 lc 6 title "A6",\
     "$DATFILE" using 6:33 with points pt 7 lc 7 title "A7",\
     "$DATFILE" using 6:37 with points pt 8 lc 9 title "A8",\
     "$DATFILE" using 6:(\$14-\$10) axis x1y2 with points pt 2 lc 2 title "P2-P1",\
     "$DATFILE" using 6:(\$18-\$10) axis x1y2 with points pt 3 lc 3 title "P3-P1",\
     "$DATFILE" using 6:(\$22-\$10) axis x1y2 with points pt 4 lc 4 title "P4-P1",\
     "$DATFILE" using 6:(\$26-\$10) axis x1y2 with points pt 5 lc 5 title "P5-P1",\
     "$DATFILE" using 6:(\$30-\$10) axis x1y2 with points pt 6 lc 6 title "P6-P1",\
     "$DATFILE" using 6:(\$34-\$10) axis x1y2 with points pt 7 lc 7 title "P7-P1",\
     "$DATFILE" using 6:(\$38-\$10) axis x1y2 with points pt 8 lc 9 title "P8-P1"

#plot "$DATFILE" using 6:9  with points pt 1 title "A1",\
#     "$DATFILE" using 6:13 with points pt 2 title "A2",\
#     "$DATFILE" using 6:(\$10-\$14) axis x1y2 with points pt 1 title "P1-P2"


#set term wxt size 1280,1024
#replot
exit 

EOF

gnuplot $PLOTFILE 
