#!/bin/bash
#
# daily_spectrum.sh 
# 
# Generate daily 3D spectrum plots
#
#     Terry Bullett  CIRES
#     23 Feb 2011
#
# Command Line Parameters
# Optional switches
# -s station Select this station 
# -y year    Select this year  (must be 4 digits!)
# -d day     Select this day (must be 3 digits!)
# -p RIQARGS Optional arguments for riq-ionogram
#
# Requires
#    
# readri2 Version 2.06+
#
APPNAME=daily_spectrum
TFMT='%F %T %Z'
YYYY=$(date -u +%Y)
DDD=$(date -u +%j)
LOGFILE="$HOME/log/$APPNAME-$YYYY-$DDD.log"
ERRLOG="$HOME/log/$APPNAME-$YYYY-$DDD.errlog"
TMPDIR="$HOME/tmp/$APPNAME"
#TMPDIR="/data/tmp/$APPNAME"
# For mirrion
DATADIR="/nfs/MIDS/data/public"
#
REALTIME=$(date  -u +"$TFMT" )
mkdir -p $TMPDIR
# declare -i YEAR DAY 
#
STATIONLIST='WI937'
YEARLIST=$(date -u +%Y --date yesterday)
DAYLIST=$(date -u +%j --date yesterday)

while getopts 'D:d:hr:s:y:' OPTION
do
   case $OPTION in
      D) DATADIR="$OPTARG"
         ;; 
      d) DAYLIST="$OPTARG"
         ;;
      h) echo "$APPNAME  Version 0.10"
         echo "Generates daily spectrum plots"
         echo "Optional Arguments:"
	 echo "  -D  DATADIR  Location of the data"
         echo "  -d  Day#. May be a list must be 3 digits Default is yesterday"
         echo "  -h  This message"
	 echo "  -p  RIQARGS  Optional argument(s) passed to riq_ionogram (via spectrumplot)"
         echo "  -s  Station. May be a list. Default is $STATIONLIST"
         echo "  -y  Year.  May be a list, must be 4 digits.  Default is yesterday"
         exit 0
         ;;
      r) RIQARGS="$OPTARG"
         ;; 
      s) STATIONLIST="$OPTARG"
         ;;
      y) YEARLIST="$OPTARG"
         ;;
   esac
done
# Remove the switches from the list.
shift $((OPTIND - 1))

#

#
PID=$$
#
for STATION in $STATIONLIST ; do
   for YEAR in $YEARLIST ; do
      for DAY in $DAYLIST ; do  
         REALTIME=$(date  -u +"$TFMT" )
         echo "$REALTIME - $APPNAME - INFO - Start movie for $STATION $YEAR $DAY"  >> $LOGFILE
#
         INDIR="$DATADIR/$STATION"/individual/"$YEAR/$DAY"'/raw'
#
         if [ ! -d $INDIR ] ; then
            echo "$REALTIME - $APPNAME - ERROR - Source directory $INDIR does not exist"  >> $LOGFILE
            exit -1
         fi 
         OUTDIR="$DATADIR/$STATION"/daily/"$YEAR/$DAY"'/misc'
         mkdir -p "$OUTDIR"
#        Generate the indvidual spectra.  This takes a 'long time'
         find $INDIR -type f -iname "$STATION_*RIQ*" -exec spectrumplot.sh -r "$RIQARGS" {}  \; > /dev/null  2>>$ERRLOG  
#
         SAMFILE="$OUTDIR/$STATION"_"$YEAR$DAY".sam
	 echo $SAMFILE
	 cat $INDIR/*.sam > $SAMFILE 2>>$ERRLOG
	 SPECTRA=$(spectrumplot-3D.sh $SAMFILE)
	 echo $SPECTRA 
#	 
	 OUTDIR="$DATADIR/$STATION"/daily/"$YEAR/$DAY"'/image'
         mkdir -p "$OUTDIR"
         mv $SPECTRA $OUTDIR/  2>>$ERRLOG
#
#        Save the individual spectral images
	 OUTDIR="$DATADIR/$STATION"/individual/"$YEAR/$DAY"'/image'
         mkdir -p "$OUTDIR"
         mv $INDIR/*sam.png  $OUTDIR/  2>>$ERRLOG
#        Delete the source files
         rm $INDIR/*.sam  2>>$ERRLOG
#	 
         REALTIME=$( date -u +"$TFMT")
         echo "$REALTIME - $APPNAME - INFO - Finished spectrum plot for $STATION $YEAR $DAY"  >> $LOGFILE
      done  # Days
   done     # Years
done        # Stations

exit 0

