SUBROUTINE DPDT(NPULSET,MASKRX,KFRQ,PRI,  Ionogram) 
!
! T. Bullett  01Jul12
!
! Compute the rate of change in phase with time across the pulset for each range gate.
!
IMPLICIT NONE
INCLUDE 'sct.f90' 
INCLUDE 'pct.f90'
INCLUDE 'pri.f90'     ! PRI
INCLUDE 'ionogram.f90'  ! Ionogram
!
! I/O variables
TYPE(IonogramType),INTENT(OUT):: Ionogram  
INTEGER,INTENT(IN) :: NPULSET
INTEGER,INTENT(IN) :: KFRQ          ! Frequency index in the ionogram
INTEGER(KIND=2),INTENT(IN):: MASKRX ! Receiver antenna selection mask
TYPE(PRItype),INTENT(IN) :: PRI(NPULSET)     ! The raw data
! 
REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: TWOPI=2.0*PI
REAL,PARAMETER :: DEGperRAD = 180.0/PI
INTEGER,PARAMETER:: NMAX=128
!
INTEGER:: I,IRG,IRX,IPS,NAVE
REAL:: RI,RQ,DP0,DPAVE,DPVAR,SCALE
REAL,DIMENSION(NPULSET,8):: PHASE
REAL,DIMENSION(NMAX):: DP
!
! Scale factor to convert raw phase differences (radians per PRI) into Cycles per Second (Hz)
!
SCALE = 1.0E6/(TWOPI*Ionogram%SCT%timing%pri)
!
! Extract the phases from the I/Q data
DO IRG=1, PRI(1)%GateCount                 ! Each range gate
   PHASE=0.0
   DO IPS=1,NPULSET
      DO IRX=1, PRI(1)%ReceiverCount          ! Each Receiver
         RI=REAL(PRI(IPS)%Ascan(IRX,IRG))
         RQ=AIMAG(PRI(IPS)%Ascan(IRX,IRG))
         PHASE(IPS,IRX)=ATAN2(RQ,RI)
      ENDDO     ! Receivers
   ENDDO        ! Pulses
   NAVE=0
   DP0=0.0
!   write(*,*) "IRG= ",IRG
!   write(*,'(8F6.0)') ((DEGperRAD*PHASE(IPS,IRX),IRX=1,8),IPS=1,NPULSET) 

   DO IPS=2,NPULSET
      DO IRX=1, PRI(1)%ReceiverCount          ! Each Receiver
!        Determine if this receiver is masked out; 2^(IRX-1) is 0 skips the receiver
         IF (.NOT.BTEST(MASKRX, IRX-1)) CYCLE
         NAVE=MIN(NAVE+1,NMAX)
         DP0=PHASE(IPS,IRX)-PHASE(IPS-1,IRX)
!        Unwrap -180/+180 degree phase ambiguity 
         IF (DP0.GT.PI) DP0=DP0-TWOPI
         IF (DP0.LT.-PI) DP0=TWOPI+DP0
         DP(NAVE)=DP0 * SCALE  !  Scaled to Hz
!         write(*,'(3I3,F6.1)') IRG,IPS,IRX,DEGperRAD*DP0
      ENDDO     ! Receivers
   ENDDO        ! Pulses
!  Mean and variance of the phase differences
   DPAVE=0.0
   DO I = 1, NAVE
      DPAVE = DPAVE + DP(I)
   ENDDO
   IF (NAVE.GT.0) DPAVE = DPAVE/REAL(NAVE)
!
   DPVAR=0.0
   DO I = 1, NAVE
      DPVAR = DPVAR + (DP(I) - DPAVE)**2
   ENDDO
   IF (NAVE.GT.1) DPVAR=DPVAR/REAL(NAVE-1)
!
   Ionogram%DPDT(IRG,KFRQ) = DPAVE
   Ionogram%DPDT_VAR(IRG,KFRQ) = DPVAR

ENDDO  ! Range Gates
!
RETURN
END SUBROUTINE DPDT
