SUBROUTINE DumpSCT(SCT)
!
! T. Bullett   CIRES     04Nov08
! g95 
!
! Dump the values in a Sounding Configuration Table (SCT) from the VIPIR RIQ files
! Version 1.2 of the RIQ format 
!
!
! 13Feb09 TWB  Removed the TrimNull() feature from DumpSCT() into a separate routine.  
!
USE misc_utils
IMPLICIT NONE
INCLUDE 'sct.f90'
INTEGER :: I,J,K 
LOGICAL :: VERBOSE=.FALSE.
!
!
TYPE(SCTtype):: SCT
!
!
WRITE(*,105) ''  
WRITE(*,105) 'GENERAL:'
WRITE(*,100) 'sct.magic:', SCT%magic 
WRITE(*,101) 'sct.sounding_table_size:', SCT%sounding_table_size 
WRITE(*,101) 'sct.pulse_table_size:', SCT%pulse_table_size 
WRITE(*,101) 'sct.raw_data_size:', SCT%raw_data_size 
WRITE(*,102) 'sct.struct_version:', SCT%struct_version
WRITE(*,101) 'sct.start_year:', SCT%start_year
WRITE(*,101) 'sct.start_daynumber:', SCT%start_daynumber
WRITE(*,101) 'sct.start_month:', SCT%start_month
WRITE(*,101) 'sct.start_day:', SCT%start_day
WRITE(*,101) 'sct.start_hour:', SCT%start_hour
WRITE(*,101) 'sct.start_minute:',  SCT%start_minute
WRITE(*,101) 'sct.start_second:', SCT%start_second
WRITE(*,101) 'sct.start_epoch:', SCT%start_epoch
WRITE(*,104) 'sct.readme:', TRIM(SCT%readme)
WRITE(*,104) 'sct.user:',TRIM(SCT%user)
WRITE(*,105) ''
!
! Station
WRITE(*,105) 'STATION:'
WRITE(*,104) 'sct.station.file_id', TRIM(SCT%station%file_id)
WRITE(*,103) 'sct.station.ursi_id', TRIM(SCT%station%ursi_id)
WRITE(*,104) 'sct.station.rx_name', TRIM(SCT%station%rx_name)
WRITE(*,102) 'sct.station.rx_latitude', SCT%station%rx_latitude
WRITE(*,102) 'sct.station.rx_longitude', SCT%station%rx_longitude
WRITE(*,102) 'sct.station.rx_altitude', SCT%station%rx_altitude
WRITE(*,101) 'sct.station.rx_count', SCT%station%rx_count
WRITE(*,105) 'sct.station.'
WRITE(*,112) 'rx_antenna_type', 'rx_position X Y Z','rx_direction X Y Z', 'rx_height', 'rx_cable_length'  
K=SCT%station%rx_count
K=MAX(1,MIN(K,32))
IF (VERBOSE) K=32
DO J=1,K
   WRITE(*,109) TRIM(SCT%station%rx_antenna_type(J)), (SCT%station%rx_position(I,J),I=1,3), &
        (SCT%station%rx_direction(I,J),I=1,3), SCT%station%rx_height(J), SCT%station%rx_cable_length(J) 
ENDDO
WRITE(*,102) 'sct.station.frontend_atten', SCT%station%frontend_atten
WRITE(*,104) 'sct.station.tx_name', TRIM(SCT%station%tx_name)
WRITE(*,102) 'sct.station.tx_latitude', SCT%station%tx_latitude
WRITE(*,102) 'sct.station.tx_longitude', SCT%station%tx_longitude
WRITE(*,102) 'sct.station.tx_altitude', SCT%station%tx_altitude
WRITE(*,108) 'sct.station.tx_vector'
WRITE(*,109) TRIM(SCT%station%tx_antenna_type), (SCT%station%tx_vector(I),I=1,3) 
WRITE(*,102) 'sct.station.tx_height', SCT%station%tx_height
WRITE(*,102) 'sct.station.tx_cable_length', SCT%station%tx_cable_length
WRITE(*,101) 'sct.station.drive_band_count',SCT%station%drive_band_count
WRITE(*,105) 'sct.station.'
WRITE(*,107) 'drive_band_bounds', 'drive_band_bounds','drive_band_atten'
K=SCT%station%drive_band_count
K=MAX(1,MIN(K,64))
IF (VERBOSE) K=64
DO J=1,K
   WRITE(*,116) SCT%station%drive_band_bounds(1,J), SCT%station%drive_band_bounds(2,J), &
                SCT%station%drive_band_atten(J)
ENDDO
WRITE(*,101) 'sct.station.rf_control',SCT%station%rf_control
WRITE(*,104) 'sct.station.ref_type',TRIM(SCT%station%ref_type)
WRITE(*,104) 'sct.station.clock_type',TRIM(SCT%station%clock_type)
WRITE(*,104) 'sct.station.user', TRIM(SCT%station%user)
WRITE(*,105) '' 
!
! Timing
WRITE(*,105) 'TIMING:'
WRITE(*,104) 'sct.timing.file_id', TRIM(SCT%timing%file_id)
WRITE(*,102) 'sct.timing.pri', SCT%timing%pri
WRITE(*,101) 'sct.timing.pri_count', SCT%timing%pri_count
WRITE(*,101) 'sct.timing.ionogram_count', SCT%timing%ionogram_count
WRITE(*,102) 'sct.timing.holdoff', SCT%timing%holdoff
WRITE(*,102) 'sct.timing.range_gate_offset',SCT%timing%range_gate_offset
WRITE(*,101) 'sct.timing.gate_count', SCT%timing%gate_count
WRITE(*,102) 'sct.timing.gate_start',SCT%timing%gate_start
WRITE(*,102) 'sct.timing.gate_end', SCT%timing%gate_end
WRITE(*,102) 'sct.timing.gate_step', SCT%timing%gate_step
WRITE(*,102) 'sct.timing.data_start', SCT%timing%data_start
WRITE(*,102) 'sct.timing.data_width', SCT%timing%data_width
WRITE(*,101) 'sct.timing.data_baud_count', SCT%timing%data_baud_count
WRITE(*,104) 'sct.timing.data_wave_file', TRIM(SCT%timing%data_wave_file)
WRITE(*,105) 'sct.timing.data_baud'
K=SCT%timing%data_baud_count
K=MAX(1,MIN(K,1024))
IF (VERBOSE) K=1024
DO I=1,K
   WRITE(*,110)  I, SCT%timing%data_baud(I)
ENDDO
WRITE(*,101) 'sct.timing.data_pairs', SCT%timing%data_pairs
WRITE(*,102) 'sct.timing.cal_start', SCT%timing%cal_start
WRITE(*,102) 'sct.timing.cal_width', SCT%timing%cal_width
WRITE(*,101) 'sct.timing.cal_baud_count', SCT%timing%cal_baud_count
WRITE(*,104) 'sct.timing.cal_wave_file', TRIM(SCT%timing%cal_wave_file)
WRITE(*,105) 'sct.timing.cal_baud'
K=SCT%timing%cal_baud_count
K=MAX(1,MIN(K,1024))
IF (VERBOSE) K=1024
DO I=1,K
   WRITE(*,110)  I, SCT%timing%cal_baud(I)
ENDDO
WRITE(*,101) 'sct.timing.cal_pairs', SCT%timing%cal_pairs
WRITE(*,104) 'sct.timing.user', TRIM(SCT%timing%user)
WRITE(*,105) '' 
!
! Frequency
WRITE(*,105)'FREQUENCY:'
WRITE(*,104) 'sct.frequency.file_id', TRIM(SCT%frequency%file_id)
WRITE(*,102) 'sct.frequency.base_start',SCT%frequency%base_start
WRITE(*,102) 'sct.frequency.base_end',SCT%frequency%base_end
WRITE(*,101) 'sct.frequency.base_steps',SCT%frequency%base_steps
WRITE(*,101) 'sct.frequency.tune_type',SCT%frequency%tune_type
WRITE(*,105) 'sct.frequency.base_table'
K=SCT%frequency%base_steps
K=MAX(1,MIN(K,8192))
IF (VERBOSE) K=8192
DO I=1,K
   WRITE(*,111)  I, SCT%frequency%base_table(I)
ENDDO
WRITE(*,102) 'sct.frequency.linear_step',SCT%frequency%linear_step
WRITE(*,102) 'sct.frequency.log_step',SCT%frequency%log_step
WRITE(*,104) 'sct.frequency.freq_table_id', TRIM(SCT%frequency%freq_table_id)
WRITE(*,101) 'sct.frequency.tune_steps',SCT%frequency%tune_steps
WRITE(*,101) 'sct.frequency.pulse_count',SCT%frequency%pulse_count
WRITE(*,105) 'sct.frequency.pulse_pattern'
K=SCT%frequency%pulse_count
K=MAX(1,MIN(K,256))
IF (VERBOSE) K=256
DO I=1,K
   WRITE(*,115)  I, SCT%frequency%pulse_pattern(I)
ENDDO
WRITE(*,102) 'sct.frequency.pulse_offset',SCT%frequency%pulse_offset
WRITE(*,101) 'sct.frequency.ramp_steps',SCT%frequency%ramp_steps
WRITE(*,101) 'sct.frequency.ramp_repeats',SCT%frequency%ramp_repeats
WRITE(*,105) 'sct.frequency.drive_table'
K=SCT%frequency%base_steps
K=MAX(1,MIN(K,8192))
IF (VERBOSE) K=8192
DO I=1,K
   WRITE(*,113)  I, SCT%frequency%drive_table(I)
ENDDO 
WRITE(*,104) 'sct.frequency.user', TRIM(SCT%frequency%user)
WRITE(*,105) '' 
!
! Receiver
WRITE(*,105)'RECEIVER:'
WRITE(*,104) 'sct.receiver.file_id', TRIM(SCT%receiver%file_id)
WRITE(*,101) 'sct.receiver.rx_chan',SCT%receiver%rx_chan
WRITE(*,105) 'sct.receiver.rx_map'
K=SCT%receiver%rx_chan
K=MAX(1,MIN(K,32))
IF (VERBOSE) K=32
DO I=1,K
   WRITE(*,115)  I, SCT%receiver%rx_map(I)
ENDDO
WRITE(*,101) 'sct.receiver.word_format',SCT%receiver%word_format
WRITE(*,101) 'sct.receiver.cic2_dec',SCT%receiver%cic2_dec
WRITE(*,101) 'sct.receiver.cic2_interp',SCT%receiver%cic2_interp
WRITE(*,101) 'sct.receiver.cic2_scale',SCT%receiver%cic2_scale
WRITE(*,101) 'sct.receiver.cic5_dec',SCT%receiver%cic5_dec
WRITE(*,101) 'sct.receiver.cic5_scale',SCT%receiver%cic2_scale
WRITE(*,104) 'sct.receiver.rcf_type',TRIM(SCT%receiver%rcf_type)
WRITE(*,101) 'sct.receiver.rcf_dec',SCT%receiver%rcf_dec
WRITE(*,101) 'sct.receiver.rcf_taps',SCT%receiver%rcf_taps
WRITE(*,105) 'sct.receiver.coefficients'
K=SCT%receiver%rcf_taps
K=MAX(1,MIN(K,160))
IF (VERBOSE) K=160
DO I=1,K
   WRITE(*,115)  I, SCT%receiver%coefficients(I)
ENDDO
WRITE(*,104) 'sct.receiver.user', TRIM(SCT%receiver%user)
WRITE(*,105) '' 
!
! Exciter
WRITE(*,105)'EXCITER:'
WRITE(*,104) 'sct.exciter.file_id', TRIM(SCT%exciter%file_id)
WRITE(*,101) 'sct.exciter.cic_scale',SCT%exciter%cic_scale
WRITE(*,101) 'sct.exciter.cic2_dec',SCT%exciter%cic2_dec
WRITE(*,101) 'sct.exciter.cic2_interp',SCT%exciter%cic2_interp
WRITE(*,101) 'sct.exciter.cic5_interp',SCT%exciter%cic5_interp
WRITE(*,104) 'sct.exciter.rcf_type',TRIM(SCT%exciter%rcf_type)
WRITE(*,101) 'sct.exciter.rcf_taps',SCT%exciter%rcf_taps
WRITE(*,101) 'sct.exciter.rcf_taps_phase',SCT%exciter%rcf_taps_phase
WRITE(*,105) 'sct.exciter.coefficients'
K=SCT%exciter%rcf_taps
K=MAX(1,MIN(K,160))
IF (VERBOSE) K=160
DO I=1,K
   WRITE(*,115)  I, SCT%exciter%coefficients(I)
ENDDO
WRITE(*,104) 'sct.exciter.user', TRIM(SCT%exciter%user)
WRITE(*,105) '' 
!
! Monitor
WRITE(*,105) 'MONITOR:'
WRITE(*,117) 'sct.monitor.balun_currents', SCT%monitor%balun_currents
WRITE(*,117) 'sct.monitor.balun_status', SCT%monitor%balun_status
WRITE(*,117) 'sct.monitor.front_end_status', SCT%monitor%front_end_status
WRITE(*,117) 'sct.monitor.receiver_status', SCT%monitor%receiver_status
WRITE(*,117) 'sct.monitor.exciter_status', SCT%monitor%exciter_status

WRITE(*,104) 'sct.monitor.user', TRIM(SCT%exciter%user)


RETURN

100 FORMAT ('#',1X,A,T33,'0x',Z8.8)
101 FORMAT ('#',1X,A,T31,I12)
102 FORMAT ('#',1X,A,T31,F12.2)
103 FORMAT ('#',1X,A,T31,A12)
104 FORMAT ('#',1X,A,T31,A)
105 FORMAT ('#',1X,A)
107 FORMAT ('#',1X,3A20)
108 FORMAT ('#',1X,A)
109 FORMAT ('#',1X,A32,6F8.2,F12.2, F18.2)
110 FORMAT ('#',1X,I5,' ( ',F5.2,', ',F5.2,' )')
111 FORMAT ('#',1X,I5,1X,F14.6)
112 FORMAT ('#',1X,A32,2A24,A12,A18)
113 FORMAT ('#',1X,I5,1X,F5.1)
! 114 FORMAT (512('#',16(1X,F5.1)/))
115 FORMAT ('#',1X,I5,1X,I5)
116 FORMAT ('#',1X,3F20.6)
117 FORMAT ('#',1X,A,T31,8(1X,Z8.8))


END
