!
! Define a structure for an Ionogram that contains various computed data types
! and various metadata needed by netCDF format files.
!
! T. Bullett   13Feb09   g95
!
!  SCT Version 1.21
!  License: Public Domain
!
!  Note:  The including program also has to include the SCT typedef
!
! History:
! 04 Oct 10 TWB  Extensively modified to add NGDC metadata and decode more SCT entries.
!
INTEGER,PARAMETER :: MAXFRQ=3000,MAXRNG=4096,MAXDOPP=16
!
TYPE::IonogramType
TYPE(SCTtype):: SCT
!
! SCT Entries copied to the Ionogram structure
CHARACTER(LEN=5)  :: URSICODE
CHARACTER(LEN=64) :: Station 
INTEGER           :: Year, DayNumber, Month, Day, Hour, Minute, Second
REAL              :: Latitude, Longitude, Altitude, StructVersion
CHARACTER(LEN=128):: Readme, User
INTEGER           :: NRANGE, NFREQ, NDOPP
INTEGER(KIND=4)   :: EpochTime
REAL:: PRI, RangeGateOffset, GateStart, GateEnd, GateStep, RANGE0
REAL:: FreqStart,FreqEnd,LinearStep,LogStep
INTEGER:: TuneType,FreqSteps, GateCount
!
! Computed or added information
REAL:: MagLat,MagLon,MagDip,GyroFreq
REAL,DIMENSION(MAXRNG,MAXFRQ) :: TPower,OPower,XPower  ! Total, O-mode, X mode ionogram powers
REAL,DIMENSION(MAXRNG,MAXFRQ) :: DPDT,DPDT_Var, AZ,ZN !  Dopper,Azimuth and Zenith  
REAL,DIMENSION(MAXFRQ) :: Frequency
REAL,DIMENSION(MAXRNG) :: Range
REAL,DIMENSION(MAXFRQ) :: Time
REAL,DIMENSION(MAXFRQ) :: TxDrive
REAL,DIMENSION(MAXFRQ) :: TNoiseLevel,TPeakLevel,TMinLevel,TPeakRange,TMinRange
REAL,DIMENSION(MAXFRQ) :: ONoiseLevel,OPeakLevel,OMinLevel,OPeakRange,OMinRange
REAL,DIMENSION(MAXFRQ) :: XNoiseLevel,XPeakLevel,XMinLevel,XPeakRange,XMinRange
INTEGER,DIMENSION(MAXFRQ) :: NumAve                          ! # of samples averaged (Normalization factor) 
! COMPLEX,DIMENSION(MAXDOPP,MAXRNG,MAXFRQ) :: Doppler     !  This is too large  Rewrite to use allocatable arrays 
! Presence of data array flags for NetCDF
INTEGER(KIND=1)::HasTotalPower,HasOModePower,HasXModePower,HasVLoS,HasDoppler,HasSPGR,HasZenith,HasAzimuth,HasCoherence
!
! NetCDF Metadata Attributes
CHARACTER(LEN=16):: Format_Version,CdmDataType
CHARACTER(LEN=10):: GeospatialLatMin,GeospatialLonMin,GeospatialVerticalMin
CHARACTER(LEN=10):: GeospatialLatMax,GeospatialLonMax,GeospatialVerticalMax
CHARACTER(LEN=10):: GeospatialLatUnits,GeospatialLonUnits,GeospatialVerticalUnits
CHARACTER(LEN=10):: GeospatialLatResolution,GeospatialLonResolution,GeospatialVerticalResolution
CHARACTER(LEN=10):: GeospatialVerticalPositive,TimeCoverageDuration,TimeCoverageResolution
CHARACTER(LEN=24):: DateCreated,DateModified,DateIssued,TimeCoverageStart,TimeCoverageEnd
CHARACTER(LEN=64):: Identifier,NamingAuthority,Title,Experiment,Instrument,Mode,CreatorName,Institution,License
CHARACTER(LEN=64):: CreatorEmail,PublisherEmail,PublisherName,Conventions,MetadataConventions,MagneticModel,Keywords
CHARACTER(LEN=64):: Project,Acknowledgment,ProcessingLevel,ContributorName,ContributorRole
CHARACTER(LEN=128):: StandardNameVocabulary,CreatorUrl,PublisherUrl
CHARACTER(LEN=256):: History,Summary,Comment,MetadataLink,KeywordsVocabulary

END TYPE IonogramType
