SUBROUTINE MPANOISE(RGT1,RGT2,NRGT,NOISEMOD,PDFOUT,FREQ,ASCAN,    AMPMIN,MINRGT,AMPMAX,MAXRGT,AMPMPA)
!
!     Calculate the Most Probable Amplitude noise value in the IONOGRAM().
!
!     Data are expected in the range of 0 to 120 and will be binned in 1 dB bins.
!     The resolution, range and number of bins is hard-coded.  Sorry.
!
!     17Sep08  Terry Bullett
!              Converted pervious F77 version to F95
!              Increased resolution of the PDF.  Most noise is +/- 5 dB
!     01Mar09  TWB  Extended the noise level to be above the MPA by a level whose
!                   probability is 1/e that of the peak.  
!                   (probably should be 1/e [1/2.718]) 
!     08Mar09  TWB Included the range bin of the AMPMIN and AMPMAX values.
!                  Passed the range gate span as parameters
!     14May09  TWB Added NOISEMOD parameter.  This factor between -1. and 1. is
!                  used to determine the probability relative to the MPA of the
!                  desired noise value.  Values < 0 mean the noise is below MPA.
!                  Dissallowed MPA values of 0.  These are below receiver threshold.
!     26May13  TWB Added a switch to output the underlying PDF statistics.
!     02Jun13  TWB Modified PDF bounds for floating point data   
! 
!     ToDo:  Check for multi-modal distributions
!            Pass PDF in and out for further analysis
!
!
IMPLICIT NONE
INTEGER,PARAMETER :: MINBIN=-200, MAXBIN=1200
!
!  Passed Parameters
INTEGER,INTENT(IN) :: RGT1,RGT2,NRGT
REAL,INTENT(IN) :: NOISEMOD,FREQ
REAL,INTENT(IN),DIMENSION(NRGT):: ASCAN
REAL,INTENT(OUT) ::  AMPMIN,AMPMAX,AMPMPA
INTEGER,INTENT(OUT) :: MINRGT,MAXRGT
LOGICAL:: PDFOUT
!
! Local Variables
REAL :: X,XMIN,XMAX,DX,NMM
INTEGER :: I,NS,NE,IX,NTOTAL,IRNG,DI
INTEGER:: PEAKVAL,PEAKCOUNT,DOWNCOUNT,NOISEVAL
INTEGER,DIMENSION(MINBIN:MAXBIN):: NX
!
XMIN = -20.0
XMAX = 160.0
DX =   1.0
!
! Determine the bin #'s for the limits
NS = INT(XMIN/DX)
NE = INT(XMAX/DX)
!
!
! Place the counts in the arrays
!
NTOTAL=0
I=0
NX=0
!
AMPMIN =  1.0E30
AMPMAX = -1.0E30
DO IRNG = RGT1, RGT2
   X=ASCAN(IRNG)
   IF (X.GT.AMPMAX) THEN
      AMPMAX=X
      MAXRGT=IRNG
   ENDIF
   IF (X.LT.AMPMIN) THEN
      AMPMIN=X
      MINRGT=IRNG
   ENDIF 
   IX = NINT(X/DX)
   IX = MAX(MIN(IX,NE),NS)
   NX(IX) = NX(IX) + 1
!db    write(*,*) IRNG,X,DX,IX,NX(IX) 
   NTOTAL = NTOTAL + 1
ENDDO  ! Range Gates
!
! Find the MPA in the NX() counts array
PEAKVAL = 0
PEAKCOUNT = 0
DO I = NS,NE
  IF (NX(I).GT.PEAKCOUNT) THEN
     PEAKCOUNT = NX(I)
     PEAKVAL = I
   ENDIF
ENDDO

! Determine the value above (or below if NOISEMOD is negative) the peak where 
! the count has dropped to NOISEMOD of its peak value
!
NMM=MIN(1.0, ABS(NOISEMOD))
NOISEVAL=PEAKVAL
DOWNCOUNT = NINT(REAL(PEAKCOUNT)*NMM)
I = PEAKVAL
DI=NINT(SIGN(1.0, NOISEMOD))
DO WHILE ((NX(I).GT.DOWNCOUNT).AND.(I.LT.NE).AND.(I.GT.NS))
   I = I + DI
ENDDO   
NOISEVAL=I
AMPMPA = REAL(NOISEVAL)*DX
!
! Output the PDF data
IF (PDFOUT) THEN
!   WRITE(*,101) "Freq","RGT1","RGT2","Min","MPA","Max","Num"
!   WRITE(*,103) FREQ,RGT1,RGT2,AMPMIN,AMPMPA,AMPMAX,NTOTAL
!   WRITE(*,*)
   WRITE(*,101) "Freq","Bin","Val","Count"  
   WRITE(*,102)  (FREQ,I,DX*I,NX(I),I=NS,NE)
   WRITE(*,*)
ENDIF
101 FORMAT ('#',A7,10A8)
102 FORMAT (F8.2,I8,F8.1,I8)
103 FORMAT (F8.2,2I8,3F8.1,I8)
RETURN
END


