#!/bin/bash
#
# powerplot -- Generate power plots for Scion HF radar data
# using read_riq Version 0.2 
# T. Bullett   17Sep08 
# This code plots raw data from each receiver stacked on top of each other.
# Warning: These raw data plots can create large temporary files which take 
#          a long time to create and plot.
#
RIQFILE=$1
BFN=$(basename $RIQFILE .RIQ)
DIR=$(dirname $RIQFILE) 
TMPDIR="$HOME/tmp/riq"
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/

# Output and temp files
mkdir -p $TMPDIR
TXTFILE=$DIR/$BFN"-s.dat"
PWRIMG=$DIR/$BFN"-s_PWR.png"
PLOTFILE=$TMPDIR/plot.gnu
#
# Default N1=1, N2=1000
N1=${3:-1000}
N2=${4:-2000} 

TITLE=$2

echo $TXTFILE $PWRIMG  $TITLE $N1  $N2 

# This is a long process; do not repeat if already done
if [ ! -f $TXTFILE  ] ; then 
   riq_ionogram -r -bp -n1 $N1 -n2 $N2  $RIQFILE > $TXTFILE
fi 

cat <<EOF > $PLOTFILE
set pm3d map corners2color c1
unset grid 
set title "$BFN  $TITLE "
set xtics  border mirror  
set xlabel "PRI"
set ylabel 'Rx-Range'
set ytics  border mirror 
set cbrange [0:120]
set size ratio 1.3
#set term png size 1024,768 crop font arial 12 
set term png large size 2048,1536 crop font arial 12 
set output "$PWRIMG"
splot "$TXTFILE" using 1:(\$5/512):11 title "Power [dB]"
set output
exit

EOF

gnuplot $PLOTFILE
# rm -f $PLOTFILE
exit 0
