#!/bin/bash
#
# powerplot -- Generate power plots and SNR plots for Scion HF radar data
# using riq_ionogram 1.2 
#
# 8 independent plots, one for each receiver
# Version 2.04  17Jan10  Terry Bullett
#
#
RIQFILE=$1
FRQSPAN="[0:12]"
BFN=$(basename $RIQFILE .RIQ)
DIR=$(dirname $RIQFILE) 
TMPDIR="$HOME/tmp/riq"
export GDFONTPATH=/usr/share/fonts/truetype/msttcorefonts/

# Output and temp files
mkdir -p $TMPDIR
PLOTFILE=$TMPDIR/plot.gnu

RXLIST='1 2 3 4 5 6 7 8'
for RX in $RXLIST ; do
   TXTFILE="$DIR/$BFN-$RX"'.dat'
   PWRIMG="$DIR/$BFN-$RX"'_PWR.png'
   SNRIMG="$DIR/$BFN-$RX"'_SNR.png'
   echo $TXTFILE $PWRIMG $SNRIMG 

#  This is a long process; do not repeat if already done
   if [ ! -f $TXTFILE  ] ; then 
      riq_ionogram -bf -i -a $RX $RIQFILE > $TXTFILE
   fi 

   cat <<EOF > $PLOTFILE
set pm3d map corners2color c1
set grid lt 22 lw 1
set xrange $FRQSPAN
set xtics 1
set mxtics 5  
set yrange[0.0:800.0] 
set ytics 100
set mytics 10   
set title "$BFN $2 Receiver $RX"
set xlabel "Frequency [MHz]"
set ylabel 'Range [km]'
set cbrange [0:90]
set cbtics 10 
set term png font arial 12 size 1024,768 crop 
set output "$PWRIMG"
splot "$TXTFILE" using 5:6:7 title "Power [dB]"
set output "$SNRIMG"
set cbrange [0:50]
splot "$TXTFILE" using 5:6:8 title "SNR [dB]"
set output
exit

EOF

   gnuplot $PLOTFILE
# rm -f $PLOTFILE
done 
exit 0
