INTEGER FUNCTION PowerSum(COHERENT,MASKRX,KFRQ,PRI,RxBeam,  Ionogram) 
!
! T. Bullett  01Oct08
!
! Add the data in PRI structure into the power sums for frequency KFRQ.
! Returns the actual number of samples added.
! RxBeam(8,2) -- Array is a complex multiplier for digital beamforming for 8 antennas and 2 beams.
!                The 2 beams defined are O and X, although more are possible.  This is site specific
!                depending on the array geometry and the nature of the amplitude and phase of the O&X
!                waves
!
! 16Feb09 TWB  Changed interface to include the entire Ionogram
! 08Mar09 TWB  Coherent sum mode allows a result of zero.  Remove values < 1.0 
!
IMPLICIT NONE
INCLUDE 'sct.f90' 
INCLUDE 'pct.f90'
INCLUDE 'pri.f90'     ! PRI
INCLUDE 'ionogram.f90'  ! Ionogram
!
! I/O variables
TYPE(IonogramType),INTENT(OUT):: Ionogram  
INTEGER,INTENT(IN) :: KFRQ          ! Frequency index in the ionogram
TYPE(PRItype),INTENT(IN) :: PRI     ! The raw data
LOGICAL,INTENT(IN) :: COHERENT      ! Flag for coherent beamforming
INTEGER(KIND=2),INTENT(IN):: MASKRX ! Receiver antenna selection mask
COMPLEX,DIMENSION(8,2),INTENT(IN) :: RxBeam ! Receiver Coefficients for the various beams.
! 
!
INTEGER:: IRG, IRX
REAL:: PSUM ,RI,RQ
COMPLEX:: CSUM,OSUM,XSUM
!
PowerSum=0
!
DO IRG=1, PRI%GateCount                 ! Each range gate
   CSUM=(0.0, 0.0)
   PSUM=0.0
   OSUM=(0.0,0.0)
   XSUM=(0.0,0.0)
   PowerSum=0
   DO IRX=1, PRI%ReceiverCount          ! Each Receiver
!     Determine if this receiver is masked out
      IF (.NOT.BTEST(MASKRX, IRX-1)) CYCLE
      PowerSum=PowerSum+1
      CSUM=CSUM + PRI%Ascan(IRX,IRG)           ! This blindly adds all receivers in phase.
      RI=REAL(PRI%Ascan(IRX,IRG))
      RQ=AIMAG(PRI%Ascan(IRX,IRG))
!      PSUM=PSUM + ABS(PRI%Ascan(IRX,IRG))
      PSUM=PSUM +  RI**2 + RQ**2 
!     Apply phase shift values for beamforming computation.
      OSUM=OSUM + (PRI%Ascan(IRX,IRG)*RxBeam(IRX,1))
      XSUM=XSUM + (PRI%Ascan(IRX,IRG)*RxBeam(IRX,2))
   ENDDO     ! Receivers
   IF (COHERENT) THEN 
      Ionogram%TPower(IRG,KFRQ)= Ionogram%TPower(IRG,KFRQ)+(REAL(CSUM)**2 + AIMAG(CSUM)**2)
   ELSE
      Ionogram%TPower(IRG,KFRQ)= Ionogram%TPower(IRG,KFRQ)+PSUM 
   ENDIF
   Ionogram%OPower(IRG,KFRQ)= Ionogram%OPower(IRG,KFRQ)+(REAL(OSUM)**2 + AIMAG(OSUM)**2)
   Ionogram%XPower(IRG,KFRQ)= Ionogram%XPower(IRG,KFRQ)+(REAL(XSUM)**2 + AIMAG(XSUM)**2)

ENDDO  ! Range Gates
!
!
RETURN
END FUNCTION PowerSum
