SUBROUTINE Read_netCDF3(NETCDFILE,   Ionogram)
!
! Program to read a NetCDF format data file which holds
! ionogram data computed from Raw ionogram data (RIQ for VIPIR, MMM,SBF,RSF from Digisondes, etc)
! 
! This program reads the netCDF file using Version 3.6 standards.
!
!
!   Terry Bullett
!   University of Colorado
!   30 March 2009 
!
! License:  Public Domain
!
! "To glorify God by exploring His universe with reverence"
! 

USE netcdf
IMPLICIT NONE
INCLUDE 'sct.f90' 
INCLUDE 'ionogram.f90'
!
CHARACTER(LEN=120),INTENT(IN)::NETCDFILE 
TYPE(IonogramType),INTENT(OUT):: Ionogram
!
! Local Variables
INTEGER:: I,IERR,NRANGE,NFREQ,NDOPP
INTEGER(KIND=1),DIMENSION(100000):: ByteSCT
REAL,PARAMETER:: Version=1.21
CHARACTER(20) :: JUNK 
!
! netCDF variables
INTEGER,DIMENSION(2) :: IonogramArray,BasePairArray,BaseDiffsArray
INTEGER,DIMENSION(4) :: CoherenceArray,DopplerArray
INTEGER:: YearID,DaynumberID,MonthID,DayID,HourID,MinuteID,SecondID,EpochID,LatitudeID,LongitudeID,AltitudeID
INTEGER:: ncid,Range_sizeID,Frequency_sizeID,Doppler_sizeID,RangeValID,FrequencyValID,TimeValID,SCTversionID
INTEGER:: TwoID,ThreeID,NumRxPairID,Range0ID,RangeGateOffsetID
INTEGER:: TxDriveID,Char08ID,Char32ID,Char64ID,URSIID,StationNameID,LinearStepID,LogStepID,NumAveID
INTEGER:: PRIID,GateCountID,GateStartID,GateEndID,GateStepID,FreqStartID,FreqEndID,FreqStepsID,TuneTypeID
INTEGER:: MagLatID,MagLonID,MagDipID,GyroFreqID 
INTEGER:: HasTotalPowerID,TotalPowerID,TotalNoiseID
INTEGER:: HasOModePowerID,OModePowerID,OModeNoiseID
INTEGER:: HasXModePowerID,XModePowerID,XModeNoiseID
INTEGER:: HasVLoSID,VLoSID
INTEGER:: HasDopplerID,DopplerID,DopplerFreqID
INTEGER:: HasSPGRID,SPGRID
INTEGER:: HasZenithID,ZenithID
INTEGER:: HasAzimuthID,AzimuthID
INTEGER:: HasCoherenceID,BaselinePairsID,BaselineDiffsID,CoherenceID 
! 
! For VIPIR
INTEGER:: SCT_sizeID,SCTID
!
! For Digisondes
INTEGER:: PREFACEID, PREF_sizeID
INTEGER:: DigisondePrefLen=77
!
!
CALL ZeroSCT(Ionogram%SCT)
!
!Open the file for reading and get it's "ncid" value
CALL CHECK( nf90_open(NETCDFILE,NF90_NOWRITE,  ncid) )

!
! Read the dimensions
! RANGE, FREQUENCY, DOPPLER
!
!
! Unlike the write routine, which must first define the file before 
! data can be written, some reads and inquiries depend on the contents
! of other data, especially the Has_xxxx  flags.  So we can't just get
! all of the ID's in one section of the code and read them all in another.
! This makes software maintenance between the read and write versions harder.
!
! Read the Variables
! 
! Station
CALL CHECK( nf90_inq_varid(ncid,"URSI",         URSIID)            )
  CALL CHECK( nf90_get_var(ncid,URSIID,         Ionogram%URSICODE))
CALL CHECK( nf90_inq_varid(ncid,"StationName",  StationNameID)     )
  CALL CHECK( nf90_get_var(ncid,StationNameID,  Ionogram%Station)  )
! Time
CALL CHECK( nf90_inq_varid(ncid,"year",         YearID)            )
  CALL CHECK( nf90_get_var(ncid,YearID,         Ionogram%Year)     )
CALL CHECK( nf90_inq_varid(ncid,"daynumber",    DaynumberID)       )
  CALL CHECK( nf90_get_var(ncid,DaynumberID,    Ionogram%DayNumber))
CALL CHECK( nf90_inq_varid(ncid,"month",        MonthID)           )
  CALL CHECK( nf90_get_var(ncid,MonthID,        Ionogram%Month)    )
CALL CHECK( nf90_inq_varid(ncid,"day",          DayID)             )
  CALL CHECK( nf90_get_var(ncid,DayID,          Ionogram%Day)      )
CALL CHECK( nf90_inq_varid(ncid,"hour",         HourID)            )
  CALL CHECK( nf90_get_var(ncid,HourID,         Ionogram%Hour)     )
CALL CHECK( nf90_inq_varid(ncid,"minute",       MinuteID)          )
  CALL CHECK( nf90_get_var(ncid,MinuteID,       Ionogram%Minute)   )
CALL CHECK( nf90_inq_varid(ncid,"second",       SecondID)          )
  CALL CHECK( nf90_get_var(ncid,SecondID,       Ionogram%Second)   )
CALL CHECK( nf90_inq_varid(ncid,"epoch",        EpochID)           )
  CALL CHECK( nf90_get_var(ncid,EpochID,        Ionogram%EpochTime))
! Sensor Location
CALL CHECK( nf90_inq_varid(ncid,"latitude",     LatitudeID)        )
  CALL CHECK( nf90_get_var(ncid,LatitudeID,     Ionogram%Latitude) )
CALL CHECK( nf90_inq_varid(ncid,"longitude",    LongitudeID)       )
  CALL CHECK( nf90_get_var(ncid,LongitudeID,    Ionogram%Longitude))
CALL CHECK( nf90_inq_varid(ncid,"altitude",     AltitudeID)        )
  CALL CHECK( nf90_get_var(ncid,AltitudeID,     Ionogram%Altitude) )
!
! Magnetic Field
CALL CHECK( nf90_inq_varid(ncid,"MagLat",       MagLatID)          )
  CALL CHECK( nf90_get_var(ncid,MagLatID,       Ionogram%MagLat)   )
CALL CHECK( nf90_inq_varid(ncid,"MagLon",       MagLonID)          )
  CALL CHECK( nf90_get_var(ncid,MagLonID,       Ionogram%MagLon)   )
CALL CHECK( nf90_inq_varid(ncid,"MagDip",       MagDipID)          )
  CALL CHECK( nf90_get_var(ncid,MagDipID,       Ionogram%MagDip)   )
CALL CHECK( nf90_inq_varid(ncid,"GyroFreq",     GyroFreqID)        )
  CALL CHECK( nf90_get_var(ncid,GyroFreqID,     Ionogram%GyroFreq) )
!
! Range Settings
CALL CHECK( nf90_inq_varid(ncid,"PRI",          PRIID)             )
  CALL CHECK( nf90_get_var(ncid,PRIID,         Ionogram%PRI)       )
CALL CHECK( nf90_inq_varid(ncid,"range_gate_offset", RangeGateOffsetID) )
  CALL CHECK( nf90_get_var(ncid,RangeGateOffsetID, Ionogram%RangeGateOffset) )
CALL CHECK( nf90_inq_varid(ncid,"gate_count",  GateCountID)        )
  CALL CHECK( nf90_get_var(ncid,GateCountID,   Ionogram%GateCount) )
CALL CHECK( nf90_inq_varid(ncid,"gate_start",  GateStartID)        )
  CALL CHECK( nf90_get_var(ncid,GateStartID,   Ionogram%GateStart) )
CALL CHECK( nf90_inq_varid(ncid,"gate_end",    GateEndID)          )
  CALL CHECK( nf90_get_var(ncid,GateEndID,     Ionogram%GateEnd)   )
CALL CHECK( nf90_inq_varid(ncid,"gate_step",   GateStepID)         )
  CALL CHECK( nf90_get_var(ncid,GateStepID,    Ionogram%GateStep)  )
CALL CHECK( nf90_inq_varid(ncid,"Range0",      Range0ID)           )
  CALL CHECK( nf90_get_var(ncid,Range0ID,      Ionogram%RANGE0)    )
! Range Array
CALL CHECK( nf90_inq_dimid(ncid,"NumRange",    Range_sizeID)                 )
  CALL CHECK( nf90_inquire_dimension(ncid,Range_sizeID, JUNK,Ionogram%NRANGE))
  NRANGE=Ionogram%NRANGE
CALL CHECK( nf90_inq_varid(ncid,"Range",       RangeValID)                   )
  CALL CHECK( nf90_get_var(ncid,RangeValID,    Ionogram%Range(1:NRANGE))     )
!
! Frequency Settings
CALL CHECK( nf90_inq_varid(ncid,"freq_start",  FreqStartID)        )
  CALL CHECK( nf90_get_var(ncid,FreqStartID,   Ionogram%FreqStart) )
CALL CHECK( nf90_inq_varid(ncid,"freq_end",    FreqEndID)          )
  CALL CHECK( nf90_get_var(ncid,FreqEndID,     Ionogram%FreqEnd)   )

CALL CHECK( nf90_inq_varid(ncid,"tune_type",   TuneTypeID)         )
  CALL CHECK( nf90_get_var(ncid,TuneTypeID,    Ionogram%TuneType)  )
CALL CHECK( nf90_inq_varid(ncid,"freq_count",  FreqStepsID)        )
  CALL CHECK( nf90_get_var(ncid,FreqStepsID,   Ionogram%FreqSteps) )
CALL CHECK( nf90_inq_varid(ncid,"linear_step", LinearStepID)       )
  CALL CHECK( nf90_get_var(ncid,LinearStepID,  Ionogram%LinearStep))
CALL CHECK( nf90_inq_varid(ncid,"log_step",    LogStepID)          )
  CALL CHECK( nf90_get_var(ncid,LogStepID,     Ionogram%LogStep)   )
! Freqeuncy Array
CALL CHECK( nf90_inq_dimid(ncid,"NumFrequency", Frequency_sizeID)                 )
  CALL CHECK( nf90_inquire_dimension(ncid,Frequency_sizeID, JUNK,Ionogram%NFREQ)  )
  NFREQ=Ionogram%NFREQ
CALL CHECK( nf90_inq_varid(ncid,"Frequency",   FrequencyValID)                    ) 
  CALL CHECK( nf90_get_var(ncid,FrequencyValID, Ionogram%Frequency(1:NFREQ))      )
!
! Following arrays are same size as Frequency
! Time Array
CALL CHECK( nf90_inq_varid(ncid,"Time",        TimeValID)                )
  CALL CHECK( nf90_get_var(ncid,TimeValID,     Ionogram%Time(1:NFREQ))   )
!
! Transmitter Drive Level
CALL CHECK( nf90_inq_varid(ncid,"TxDrive",     TxDriveID)                )
  CALL CHECK( nf90_get_var(ncid,TxDriveID,     Ionogram%TxDrive(1:NFREQ)))
!
! Normalization Factor
CALL CHECK( nf90_inq_varid(ncid,"NumAve",      NumAveID)                 )
CALL CHECK( nf90_get_var(ncid,NumAveID,        Ionogram%NumAve(1:NFREQ)) )


! Read the VIPIR SCT ID's
CALL CHECK( nf90_inq_varid(ncid, "SCT_version", SCTversionID)              )
  CALL CHECK( nf90_get_var(ncid,SCTversionID, Ionogram%SCT%struct_version ))

! SCT for VIPIR sounders
CALL CHECK( nf90_inq_dimid(ncid,"SCT_size",  SCT_sizeID)                                    )
  CALL CHECK( nf90_inquire_dimension(ncid,SCT_sizeID, JUNK,Ionogram%SCT%sounding_table_size))
! Load in the SCT
CALL CHECK( nf90_inq_varid(ncid, "SCT",             SCTID)                            )
   CALL CHECK( nf90_get_var(ncid,SCTID,  ByteSCT(1:Ionogram%SCT%sounding_table_size)) )
! ToDo : Transfer this to the SCT structure with code like this:  
! Requires:  IonogramFile, SCT structure, 
! Map the SCT into an array of bytes for the netCDF file.  This is done through a temporary file.
!ByteSCT=0 
!OPEN (UNIT=99, STATUS="SCRATCH",FORM="UNFORMATTED", ACCESS="STREAM")
!WRITE(99) Ionogram%SCT
!REWIND(99)
!READ(99,IOSTAT=IERR) (ByteSCT(I),I=1,Ionogram%SCT%sounding_table_size)
!CLOSE(99)
!
! Preface for Digisondes
CALL CHECK( nf90_inq_dimid(ncid, "PREFACE_size",  PREF_sizeID)              )
CALL CHECK( nf90_inquire_dimension(ncid,PREF_sizeID, JUNK,DigisondePrefLen) )
CALL CHECK( nf90_inq_varid(ncid, "PREFACE",            PREFACEID)           )
!
! Other brands & Models go here
!
! The 2D data arrays.  These are optional, so read them conditionally  
!
! Total Power
CALL CHECK( nf90_inq_varid(ncid, "Has_total_power",  HasTotalPowerID)                   )
   CALL CHECK( nf90_get_var(ncid,HasTotalPowerID,    Ionogram%HasTotalPower)            )
IF (Ionogram%HasTotalPower.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "total_power",   TotalPowerID)                      )
     CALL CHECK( nf90_get_var(ncid,TotalPowerID,     Ionogram%TPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "total_noise",   TotalNoiseID)                      )
     CALL CHECK( nf90_get_var(ncid,TotalNoiseID,     Ionogram%TNoiseLevel(1:NFREQ)    ) )
ENDIF 
!
! Ordinary Mode Power
CALL CHECK( nf90_inq_varid(ncid, "Has_O-mode_power", HasOModePowerID)                   )
  CALL CHECK( nf90_get_var(ncid,HasOModePowerID,     Ionogram%HasOModePower)            )
IF (Ionogram%HasOModePower .NE. 0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "O-mode_power",  OModePowerID)                      )
     CALL CHECK( nf90_get_var(ncid,OModePowerID,     Ionogram%OPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "O-mode_noise",  OModeNoiseID)                      )
     CALL CHECK( nf90_get_var(ncid,OModeNoiseID,     Ionogram%ONoiseLevel(1:NFREQ)    ) )
ENDIF
!
! eXtraordinary Mode Power
CALL CHECK( nf90_inq_varid(ncid, "Has_X-mode_power", HasXModePowerID)                   )
  CALL CHECK( nf90_get_var(ncid,HasXModePowerID,     Ionogram%HasXmodePower)            )
IF (Ionogram%HasXModePower .NE. 0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "X-mode_power",  XModePowerID)                      )
      CALL CHECK( nf90_get_var(ncid,XModePowerID,    Ionogram%XPower(1:NRANGE,1:NFREQ)) )
   CALL CHECK( nf90_inq_varid(ncid, "X-mode_noise",  XModeNoiseID)                      )
      CALL CHECK( nf90_get_var(ncid,XModeNoiseID,    Ionogram%XNoiseLevel(1:NFREQ)    ) )
ENDIF
!
! ToDo:  Actually read some of these data if they are present.
!
! LoS Velocity Data
!CALL CHECK( nf90_inq_varid(ncid, "Has_VLoS",       HasVLoSID)           )
!   CALL CHECK( nf90_get_var(ncid,HasDopplerID,     Ionogram%HasDoppler) )
!IF (Ionogram%HasVLoS.NE.0) THEN
!   CALL CHECK( nf90_inq_varid(ncid, "VLoS",        VLoSID)              )
!ENDIF
!

! Doppler Shift Data
CALL CHECK( nf90_inq_dimid(ncid,"NumDoppler",      Doppler_sizeID)            )
CALL CHECK( nf90_inquire_dimension(ncid,Doppler_sizeID, JUNK,Ionogram%NDOPP)  )
NDOPP=Ionogram%NDOPP 
CALL CHECK( nf90_inq_varid(ncid, "Has_Doppler",    HasDopplerID)              )
   CALL CHECK( nf90_get_var(ncid,HasDopplerID,     Ionogram%HasDoppler)       )
IF (Ionogram%HasDoppler.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "Doppler",     DopplerID)                 )
ENDIF
!
! Stationary Phase Group Range or Precision Group Height  
CALL CHECK( nf90_inq_varid(ncid, "Has_SPGR",      HasSPGRID)                  )
  CALL CHECK( nf90_get_var(ncid,HasSPGRID,        Ionogram%HasSPGR)           )
IF (Ionogram%HasSPGR.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "SPGR",       SPGRID)                     )
ENDIF
!
! Zenith Angle data
CALL CHECK( nf90_inq_varid(ncid, "Has_Zenith",    HasZenithID)                )
  CALL CHECK( nf90_get_var(ncid,HasZenithID,        Ionogram%HasZenith)       )
IF (Ionogram%HasZenith.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "zenith",     ZenithID)                   )
ENDIF
!
! Azimuth Angle
CALL CHECK( nf90_inq_varid(ncid, "Has_Azimuth",   HasAzimuthID)               )
   CALL CHECK( nf90_get_var(ncid,HasAzimuthID,    Ionogram%HasAzimuth)        )
IF (Ionogram%HasAzimuth.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "azimuth",    AzimuthID)                  )
ENDIF
!
! Coherence Arrays
CALL CHECK( nf90_inq_varid(ncid, "Has_Coherence",      HasCoherenceID)        )
  CALL CHECK( nf90_get_var(ncid,HasCoherenceID,        Ionogram%HasCoherence) )
IF (Ionogram%HasCoherence.NE.0) THEN
   CALL CHECK( nf90_inq_varid(ncid, "baseline_pairs",  BaselinePairsID)       )
   CALL CHECK( nf90_inq_varid(ncid, "baseline_diffs",  BaselineDiffsID)       )
   CALL CHECK( nf90_inq_varid(ncid, "coherence",       CoherenceID)           )
!  Add coherence arrays here
ENDIF
!
! Global Attributes 
! ToDo: Figure out how to read these later
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "format_version", Ionogram%Format_Version ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "title", Ionogram%Title ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "instrument", Ionogram%Instrument ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "experiment", Ionogram%Experiment ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "mode", Ionogram%Mode ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "point_of_contact", Ionogram%PoC ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "institution", Ionogram%Institution ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "email", Ionogram%Email ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "release", Ionogram%Release ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "Conventions", Ionogram%Conventions) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "description", Ionogram%Description ) )
!CALL CHECK( nf90_put_att(ncid,NF90_GLOBAL, "history", Ionogram%History) )
!
!
! Close the file.
CALL CHECK( nf90_close(ncid) )
!
!
RETURN
!
!
CONTAINS

SUBROUTINE CHECK(STATUS)
!
! Writes NetCDF errors to standard error 
!
INTEGER,INTENT(IN):: STATUS

IF ( STATUS.NE.NF90_NOERR) WRITE(0,*) "read_netCDF3 -- ERROR -- ", trim(nf90_strerror(STATUS))
RETURN
END SUBROUTINE CHECK 

END 
