SUBROUTINE ReadConfig(ConfigFile,  Ionogram)
!
! Update the Ionogram structure using metatadata from a configuration file
! specified by the calling program.
!
! Terry Bullett
! University of Colorado Boulder
!
! 01 October 2010
!
! License: Public Domain
!
! "Knowledge, Sir, should be free to all"  -- Harry Mudd
!
!
IMPLICIT NONE
INCLUDE 'sct.f90'      ! SCT
INCLUDE 'ionogram.f90' ! Processed ionogram structure
!
! Passed Variables
CHARACTER(LEN=128),INTENT(IN):: ConfigFile     ! Full path/name of input file
TYPE(IonogramType),INTENT(INOUT) :: Ionogram
!
! Local Varables
INTEGER,PARAMETER:: IU=24
INTEGER:: IERR,POS
LOGICAL:: EOF
CHARACTER(LEN=128) :: Line
CHARACTER(LEN=64) :: Keyword, Value
CHARACTER(LEN=1) :: HASH,EQUAL 

HASH='#'
EQUAL='='

OPEN(UNIT=IU,FILE=ConfigFile, FORM='FORMATTED', ACCESS='SEQUENTIAL', IOSTAT=IERR)

! Since this step is optional, all errors are non-fatal and are silently ignored.
IF (IERR.NE.0) RETURN

EOF=.FALSE.
DO WHILE (.NOT.EOF) 
   READ(IU,100,IOSTAT=IERR) Line
   IF (IERR.NE.0) THEN
      EOF=.TRUE.
      CYCLE
   ENDIF
!
!  Lines starting with a # are comments and ignored.
   IF(Keyword(1:1).EQ.HASH) CYCLE
!  Split the line across the = symbol
   POS=INDEX(Line, EQUAL)
   IF (POS.EQ.0) CYCLE   !  Invalid line
   Keyword=ADJUSTL((TRIM(Line(1 : POS-1))))
   Value=ADJUSTL(TRIM(Line(POS+1 : LEN_TRIM(Line))))
!  Strip strings to exclude values after #
   POS=INDEX(Keyword,HASH)
   IF (POS.GT.0) Keyword=Keyword(1 : POS-1)
   POS=INDEX(Value,HASH)
   IF (POS.GT.0) Value=Value(1 : POS-1)
!db   write(*,*) "DEBUG ", EOF, Keyword, Value 
!
!  Parse known Keyword--Value combinations
   SELECT CASE (TRIM(Keyword))
      CASE ("Title")
         Ionogram%Title=TRIM(Value)
      CASE ("Mode")
         Ionogram%Mode=TRIM(Value)
      CASE DEFAULT
!db         WRITE(*,*) " ReadConfig - Unknown entry - Keyword=",TRIM(Keyword), ' Value=',TRIM(Value)
         CONTINUE
   END SELECT
END DO   ! Each line in the file    


!db write(*,*) 'Mode = ', TRIM(Ionogram%Mode)

CLOSE(UNIT=IU)

RETURN

100 FORMAT (A128)
END
