PROGRAM riq2wav
!
! Generates and audio WAV format file from RIQ data,
! to enable the listening of radar data.
!
!  16Feb13  Terry Bullett   University of Colorado
!  To Explore His universe with Reverence. 
!

IMPLICIT NONE
INCLUDE 'sct.f90'
INCLUDE 'pct.f90'
INCLUDE 'pri.f90'
INCLUDE 'wav.f90'

TYPE(SCTtype) SCT
TYPE(PCTtype) PCT
TYPE(PRItype) PRI
TYPE(WAVtype) WAV
!
! Local Variables
CHARACTER(120)::INFILE,CLP,OUTFILE
LOGICAL INFOK,EOF,SKIP 
INTEGER :: IU=1,OU=2,FLIPHASE=0,N1=0,N2=0,WORDFORMAT=0
INTEGER :: K,NA,IANT,IPRI,NUMPRI,IRG,NUMRG,IRX,RGT1,RGT2,NP
INTEGER(KIND=2):: SAMPLE
REAL:: VERSION=0.1, VOLUME=100.0, RANGE1=0.0, RANGE2=0.0, RANGE0=0.0

! Functions
LOGICAL:: READSCT, READPCT, READPRI 
!
! Parse the command line
SKIP=.FALSE.
OUTFILE='RIQ.wav'
IANT=1

! Read the command line for arguments
K = IARGC() 
! Filename must exist
IF (K.LT.1) THEN
   WRITE(*,*) "riq2wav - ERROR: Input file must be specified"
   CALL EXIT(-1)
ENDIF
DO NA = 1, K
   CALL GETARG(NA,CLP)
   IF (CLP.EQ.'-h') THEN
      WRITE(*,*)' useage: riq2wav {} filename'
      WRITE(*,*)' '
      WRITE(*,'(A,F8.3)') 'Version: ',VERSION
      WRITE(*,*)'Reformat RIQ data into WAV sound file format'
      WRITE(*,*)' '
      WRITE(*,*)'Most of these are not yet enabled'
      WRITE(*,*)' -a {n} Select antenna (channel) n '
      WRITE(*,*)' -o {Outfile} Sets output file.  Default is RIQ.wav'
      WRITE(*,*)' -r1 Lower range [km]'
      WRITE(*,*)' -r2 Upper range [km]'
      WRITE(*,*)' -n1 First PRI'
      WRITE(*,*)' -n2 Last PRI'
      WRITE(*,*)' -V {volume} Volume Multiplier'
      WRITE(*,*)' Filename must be provided on the command line.'
      RETURN
   ELSE IF (SKIP) THEN
      SKIP=.FALSE.
   ELSE IF (CLP.EQ.'-a') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) IANT
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-o') THEN
      CALL GETARG(NA+1,OUTFILE)
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-r2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) RANGE2
     SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n1') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N1
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-n2') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) N2
      SKIP=.TRUE.
   ELSE IF (CLP.EQ.'-V') THEN
      CALL GETARG(NA+1,CLP)
      READ(CLP,*) VOLUME
     SKIP=.TRUE.
   ELSE
!    This is the input file name
     INFILE=CLP
   ENDIF
ENDDO  ! Command Line parsing
! 
!  Check if input file exists
INQUIRE(FILE=INFILE,EXIST=INFOK)
IF (.NOT.INFOK) THEN
   WRITE(*,*) ' riq2wav - error - Missing input file'
   CALL EXIT(-1)
ENDIF

!
!
! Set values into the headers
!
CALL ZeroSCT(SCT)
!
OPEN(UNIT=IU, FILE=INFILE, FORM="UNFORMATTED", ACCESS="STREAM")
! Read the SCT
EOF=READSCT(IU,  SCT)
IF (EOF) CALL EXIT(1)
CALL FixSCTstrings(SCT)
!
NUMPRI=SCT%timing%pri_count
NUMRG=SCT%timing%gate_count
NP=NUMPRI
IF ((N1.GT.0).OR.(N2.GT.0)) NP=N2-N1 
!
WAV%ChunkID="RIFF"
WAV%Format="WAVE"
WAV%Subchunk1ID="fmt "
WAV%SubChunk1Size=16
WAV%AudioFormat=1
WAV%NumChannels=1     ! hard wired for now
WAV%BitsPerSample=16
WAV%SubChunk2Size=NP*NUMRG* WAV%NumChannels * WAV%BitsPerSample/8
WAV%ChunkSize=36+WAV%SubChunk2Size
WAV%SampleRate=INT(1.0/(1.0E-6*SCT%timing%gate_step))  ! Should be 100,000 
WAV%ByteRate=WAV%SampleRate * WAV%NumChannels * WAV%BitsPerSample/8
WAV%BlockAlign= WAV%NumChannels * WAV%BitsPerSample/8
WAV%Subchunk2ID="data"

OPEN(UNIT=OU, FILE=OUTFILE, FORM="UNFORMATTED", ACCESS="STREAM")
write(2) WAV
!db WRITE(*,*) NUMPRI, NUMRG 
!db write(*,*) WAV 
!
! Set Range Gate Limits.
RGT1=1
RGT2=NUMRG
IF(RANGE1.GT.0) THEN
   RGT1=NINT((RANGE1-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT1=MIN(MAX(1,RGT1),NUMRG)
ENDIF
IF (RANGE2.GT.0) THEN
   RGT2=NINT((RANGE2-RANGE0)/(0.150*REAL(SCT%Timing%gate_step)))
   RGT2=MIN(MAX(1,RGT2),NUMRG)
ENDIF

IF (N1.EQ.0) N1=1
IF (N2.EQ.0) N2=NUMPRI
write (*,*) VOLUME, N1, N2, RGT1, RGT2 

DO IPRI = 1, NUMPRI 
   CALL ZeroPCT(PCT)
   EOF=READPCT(IU,  PCT)
   IF(EOF) CALL EXIT(1)
   CALL FixPCTstrings(PCT)
!  Copy relevant elements from SCT and PCT to the PRI Structure
   CALL SCTPCT2PRI(SCT,PCT,0.0, PRI )
   EOF=READPRI(IU,FLIPHASE,WORDFORMAT,  PRI )         ! Read one A-scan
   IF(EOF) CALL EXIT(-1) 
   IF ((IPRI.LT.N1).OR.(IPRI.GT.N2)) CYCLE
!  Hard-wire the output file to Receiver 1, In-phase channel. 
   DO IRG = 1, NUMRG
      SAMPLE=0
      IF ((IRG.GE.RGT1).AND.(IRG.LE.RGT2)) SAMPLE=INT(VOLUME*REAL(PRI%Ascan(IANT,IRG)))
      WRITE(OU) SAMPLE
!db      WRITE(*,*) SAMPLE  
   ENDDO  ! Range Gates
ENDDO     ! PRIs
!
CLOSE(UNIT=IU)
CLOSE(UNIT=OU)

!
END PROGRAM
