SUBROUTINE SIF (PRI)
!
! Simple Interferometry
! 10Aug11  TWB
! Compute zenith and azimuth angles for each range gate in the PRI using simple interferometry
USE misc_utils
IMPLICIT NONE

INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI

REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: DEGperRAD = 180.0/PI
!
! MapA - Antenna Map is the mapping of receiver number to the sequence needed for this algorithm.
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2 = De-aliasing pairs ; 3,4 are the outer antennas   
INTEGER,DIMENSION(2,4):: MapA
INTEGER I,IRX,IRNG,IXY
REAL RI,RQ
LOGICAL WRAPPED
REAL,DIMENSION(2):: Phi0,DPhi,Ephi
SAVE MapA
DATA MapA(1,I),I=1, 4) /6,8,2,4/
DATA MapA(2,I),I=1, 4) /5,7,1,3/

Zenith=0.0
Azimuth=0.0
Doppler=0.0
ZnErr=0.0
AzErr=0.0
DopErr=0.0
Amplitude=0.0
Phase=0.0
!
! Compute amplitude (dB) and phase (radians) for each range gate and receiver
DO IRNG=1,PRI(IPRI)%GateCount
   DO IRX = 1, PRI(IPRI)%ReceiverCount
      RI=REAL( PRI%Ascan(IRX,IRNG))
      RQ=AIMAG(PRI%Ascan(IRX,IRNG))  
      Amplitude(IRX)=10.*LOG10(RI**2 + RQ**2)
      Phase(IRX)=ATAN2(RQ,RI)
   ENDDO
!
! De-alias the phases
   DO IXY=1,2       ! Each X and Y directions independent
      Phi0(IXY) = 0.5*( Phase(MapA(IXY,1)) - Phase(MapA(IXY,2)) )
      WRAPPED=.FALSE.
      WRAPPED= (Phi0(IXY)-ABS(Phase(MapA(IXY,1)))).GT.PI/2.0) .OR. (Phi0(IXY)-ABS(Phase(MapA(IXY,2)))).GT.PI/2.0)
      IF (WRAPPED) THEN
         Phase(MapA(IXY,2)) = Phase(MapA(IXY,2)) + 2.0*PI
         Phi0(IXY) = 0.5*( Phase(MapA(IXY,1)) - Phase(MapA(IXY,2)) )
      ENDIF
   write(*,100) IRTG,IXY,  DEGperRAD*Phi0(IXY), DEGperRAD*Phase(MapA(IXY,1)), DEGperRAD*Phase(MapA(IXY,2))
   ENDDO 
ENDDO

END SUBROUTINE 


