SUBROUTINE SIMPLE_INTERFEROMETER(PRI)
!
! Simple Phasse based interferometer.
! 10Aug11  TWB
! Compute zenith and azimuth angles for each range gate in the PRI using simple interferometry on the received phases.
! This metthod treats each PRI independently.  
USE misc_utils
IMPLICIT NONE

INCLUDE 'pct.f90'   ! PCT
INCLUDE 'sct.f90'   ! SCT
INCLUDE 'pri.f90'   ! PRI

REAL,PARAMETER :: PI=4.0*ATAN(1.0)
REAL,PARAMETER :: TWOPI=2.0*PI
REAL,PARAMETER :: DEGperRAD = 180.0/PI
! 
! Arguments
TYPE(PRItype),INTENT(INOUT) :: PRI
!
! MapA - Antenna Map is the mapping of receiver number to the sequence needed for this algorithm.
! First dimension: 1=X (East) ,2=Y (North)
! Second dimension: 1,2 = De-aliasing pairs ; 3,4 are the outer antennas   
INTEGER,DIMENSION(2,4):: MapA
INTEGER I,IRX,IRG,IXY
REAL RI,RQ,DPH,P1,P2
LOGICAL WRAPPED
REAL,DIMENSION(2):: P0,DP
SAVE MapA
!
DATA (MapA(1,I),I=1, 4) /6,8,2,4/
DATA (MapA(2,I),I=1, 4) /5,7,1,3/
!
PRI%Zenith=0.0
PRI%Azimuth=0.0
PRI%Doppler=0.0
PRI%ZnErr=0.0
PRI%AzErr=0.0
PRI%DopErr=0.0
PRI%Amplitude=0.0
PRI%Phase=0.0
!
! Compute amplitude (dB) and phase (radians) for each range gate and receiver
DO IRG=1,PRI%GateCount

   DO IRX = 1, PRI%ReceiverCount
      RI=REAL( PRI%Ascan(IRX,IRG))
      RQ=AIMAG(PRI%Ascan(IRX,IRG))  
      PRI%Amplitude(IRX,IRG)=10.*LOG10(RI**2 + RQ**2)
      PRI%Phase(IRX,IRG)=ATAN2(RQ,RI)
   ENDDO
!
!db write(*,'(8F8.1)') (DEGperRAD*PRI%Phase(IRX,IRG),IRX=1,8)
! De-alias the phases
   DO IXY=1,2       ! Each X and Y directions independent.  This may cause different phase offests at the origin?
!db write(*,*) IXY, MapA(IXY,1), MapA(IXY,2) 
      P1=PRI%Phase(MapA(IXY,1),IRG)  ! Terse access to the two phases involved.
      P2=PRI%Phase(MapA(IXY,2),IRG)
      DPH = (P2-P1) 
      WRAPPED=(ABS(DPH).GT.PI)
 !     Determine which phase is closest to +/-PI.  This will be the one with the largest absolute value.
      IF (WRAPPED) THEN
         IF (ABS(P1).GT.ABS(P2)) THEN !   Phase 1 is wrapped
            PRI%Phase(MapA(IXY,1),IRG)=SIGN(TWOPI,P2)+P1
         ELSE                         !   Phase 2 is wrapped
            PRI%Phase(MapA(IXY,2),IRG)=SIGN(TWOPI,P1)+P2
         ENDIF
     ENDIF   ! Phase Wrapped
!     Fit a phase slope vs distance line using these unwrapped phases.
!     ToDo: Use the two point method instead of the slope-intercept.  More versitile.  
      P0(IXY)= 0.5*(PRI%Phase(MapA(IXY,2),IRG) + PRI%Phase(MapA(IXY,1),IRG))  ! Phase at origin for this axis.  Assumes symmetry at Wallops.
      DP(IXY)= ( PRI%Phase(MapA(IXY,2),IRG) - PRI%Phase(MapA(IXY,1),IRG) )    ! Phase slope. 
!   write(*,*) IXY, MapA(IXY,2), DEGperRAD*PRI%Phase(MapA(IXY,2),IRG) , MapA(IXY,1) , &
!              DEGperRAD*PRI%Phase(MapA(IXY,1),IRG), DEGperRAD*P0(IXY), DEGperRAD*DP(IXY)

   ENDDO ! X and Y directions
    write(*,100) (IRG, IXY, WRAPPED, MapA(IXY,2), MapA(IXY,1), DEGperRAD*PRI%Phase(MapA(IXY,2),IRG), &
                  DEGperRAD*PRI%Phase(MapA(IXY,1),IRG), PRI%Frequency, DEGperRAD*P0(IXY), DEGperRAD*DP(IXY),  IXY=1,2)
ENDDO  ! Range Gates

100 FORMAT (2(2I5, L5, 2I5, 5F8.1))
END SUBROUTINE SIMPLE_INTERFEROMETER


